@@ -8,14 +8,18 @@
 #' @param cutpoint Cutpoint
 #' @param labels Additional labels to provide as list (with entries \code{x}, \code{y}, and eventually vector \code{covar}). Unused currently. 
 #' @param data A data-frame for the \code{x} and \code{y} variables. If this is provided, 
-#' the column names can be entered directly for argument \code{x} and \code{y}
+#' the column names can be entered directly for argument \code{x}, \code{y} and \code{covar}. 
+#' For \code{covar}, should be a character vector. 
 #' @param z Assignment variable for the fuzzy case. Should be 0/1 or TRUE/FALSE variable. 
+#' @details Arguments \code{x}, \code{y} (and eventually \code{covar}) can be either given as:
+#'  * vectors (eventually data-frame for \code{covar})
+#'  * quote/character when \code{data} is also provided. For multiple \code{covar}, use a vector of characters 
 #' @return Object of class \code{rdd_data}, inheriting from \code{data.frame}
-#' @author Matthieu Stigler <\email{Matthieu.Stigler@@gmail.com}>
+#' @author Matthieu Stigler \email{Matthieu.Stigler@@gmail.com}
 #' @export
 #' @examples
 #' data(house)
-#' rd<- rdd_data(x=house$x, y=house$y, cutpoint=0)
+#' rd <- rdd_data(x=house$x, y=house$y, cutpoint=0)
 #' rd2 <- rdd_data(x=x, y=y, data=house, cutpoint=0)
 #' 
 #' # The print() function is the same as the print.data.frame:
@@ -30,7 +34,7 @@
 #'                              z=ifelse(house$x>0+rnorm(nrow(house), sd=0.05),1,0), 
 #'                              cutpoint=0)
 #' summary(rd_dat_fakefuzzy)
-
+#' @md
 
 rdd_data <- function(y, x, covar, cutpoint, z, labels, data) {
     
@@ -47,10 +51,27 @@ rdd_data <- function(y, x, covar, cutpoint, z, labels, data) {
         pf <- parent.frame()
         x <- eval(substitute(x), data, enclos = pf)  # copy from with.default
         y <- eval(substitute(y), data, enclos = pf)  # copy from with.default
-        if (hasCovar) 
-            covar <- eval(substitute(covar), data, enclos = pf)  # idem
+        if (hasCovar) {
+            ## make sure it's not already a df!?
+            class_robust <- try(class(eval(substitute(covar))), silent=TRUE)
+            if(inherits(class_robust, "try-error")) class_robust <- "quote"
+            
+            if(any(c("data.frame", "numeric") %in% class_robust)) {
+                covar_df <- covar
+            } else {
+                ## copy code from subset.data.frame
+                nl <- as.list(seq_along(data))
+                names(nl) <- names(data)
+                covar_index <- eval(substitute(covar), nl, parent.frame())
+                
+                covar_df <- data[,covar_index, drop=FALSE]  
+            }
+        }
     }
     
+    if (missing(data) & hasCovar) covar_df <- covar 
+    
+    
     ### Check y, x univariate
     k_y <- NCOL(y)
     k_x <- NCOL(x)
@@ -79,8 +100,8 @@ rdd_data <- function(y, x, covar, cutpoint, z, labels, data) {
         if (is.null(names(labels)) || !all(names(labels) %in% c("x", "y", "covar"))) 
             stop("labels should be a list with components x, and/or y, and/or covar")
         if (hasCovar) {
-            if ("covar" %in% names(labels) && length(labels$covar) != NCOL(covar)) 
-                stop("There should be ", NCOL(covar), " values (dim of covar) for component 'covar' in labels")
+            if ("covar" %in% names(labels) && length(labels$covar) != NCOL(covar_df)) 
+                stop("There should be ", NCOL(covar_df), " values (dim of covar) for component 'covar' in labels")
         }
     } else {
         labels <- list()
@@ -92,8 +113,8 @@ rdd_data <- function(y, x, covar, cutpoint, z, labels, data) {
     ## Assemble data
     rdd_dat <- data.frame(x = x, y = y)
     if (hasCovar) {
-        rdd_dat <- cbind(rdd_dat, covar)
-        if (NCOL(covar) == 1 && is.null(colnames(covar))) 
+        rdd_dat <- cbind(rdd_dat, covar_df)
+        if (NCOL(covar_df) == 1 && is.null(colnames(covar_df))) 
             colnames(rdd_dat)[3] <- covar_nam
     }
     
